package hu.kozadat.search.base.preprocessor;

import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * User: codex
 */
@Component("datePreprocessor")
public class DatePreprocessor extends QueryPreprocessor {
    private static final Pattern yearPattern = Pattern.compile("(^|[\\s()])((isz|ie)[.]?[\\s]*)?([0-9]{1,4})([./-])([0][1-9]|[1][0-2])\\5([0-9]{2})[.]?(\\b|\\B)", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);

    protected String formatDate(int year, int month, int day) {
        Date date = new GregorianCalendar(year, month - 1, day).getTime();

        return df.format(date);
    }

    public QueryPreprocessorResult preprocess(String segment, boolean appendFieldName) {
        QueryPreprocessorResult res = new QueryPreprocessorResult();
        res.setOriginalQuery(segment);
        Matcher yearMatcher = yearPattern.matcher(segment);
        StringBuffer sb = new StringBuffer();

        String fieldStr = appendFieldName ? "date:" : "";

        while (yearMatcher.find()) {
            //-- not null
            String startStr = yearMatcher.group(1);
            //-- can be null
            String timelinePartSelector = com.actiwise.einfoszab.Utils.null2String(yearMatcher.group(3)).toLowerCase();
            //-- not null
            String yearStr = yearMatcher.group(4);
            //-- not null
            String monthStr = yearMatcher.group(6);
            //-- not null
            String dayStr = yearMatcher.group(7);

            boolean isIsz = true;
            if (!"".equals(timelinePartSelector)) {
                if (BEFORE_CHRISTMAS.equals(timelinePartSelector))
                    isIsz = false;
            }

            int year = -1;
            int month = -1;
            int day = -1;
            boolean validDate = true;
            try {
                year = Integer.parseInt(yearStr);
                month = Integer.parseInt(monthStr);
                day = Integer.parseInt(dayStr);
            } catch (NumberFormatException nex) {
                validDate = false;
            }

            if (validDate) {
                MonthContainer mc = monthConversionMap.get(String.valueOf(month));
                if (mc != null) {
                    if (mc.getMonthNum() != 2) {
                        if (day > mc.getNumDays())
                            validDate = false;
                    } else {
                        if (day > (isLeapYear(year) ? mc.getNumDays() + 1 : mc.getNumDays()))
                            validDate = false;
                    }
                } else {
                    validDate = false;
                }
            }

            String yearQuery = startStr;
            if (validDate)
                yearQuery = startStr + fieldStr + formatDate(year, month, day);

            yearMatcher.appendReplacement(sb, yearQuery);
            res.setQueryModified(true);
        }
        yearMatcher.appendTail(sb);

        res.setFilteredQuery(sb.toString());
        return res;
    }
}
