package hu.kozadat.search.base.preprocessor;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * User: codex
 */
@Component("yearMonthPreprocessor")
public class YearMonthPreprocessor extends QueryPreprocessor {

    private static final Pattern yearMonthPattern = Pattern.compile("(^|[\\s()])((isz|ie)[.]?[\\s]*)?([0-9]{1,4})([.]?[\\s]*(január|február|március|április|május|június|július|augusztus|szeptember|október|november|december)|[./]([0][1-9]|[1][0-2])[.]?)\\b", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);

    private DatePrecision datePrecision;

    public DatePrecision getDatePrecision() {
        return datePrecision;
    }

    @Autowired
    public void setDatePrecision(DatePrecision datePrecision) {
        this.datePrecision = datePrecision;
    }

    protected String formatStartMonth(int year, MonthContainer mc) {
        GregorianCalendar cal = new GregorianCalendar(year, mc.getMonthNum() - 1, 1);
        Date event = cal.getTime();

        if (datePrecision != null)
            event = datePrecision.adjustDateDownwards(event);

        return df.format(event);
    }

    protected String formatEndMonth(int year, MonthContainer mc, boolean isLeapYear) {
        int day = (mc.getMonthNum() == 2 && isLeapYear) ? mc.getNumDays() + 1 : mc.getNumDays();

        GregorianCalendar cal = new GregorianCalendar(year, mc.getMonthNum() - 1, day);
        Date event = cal.getTime();

        if (datePrecision != null)
            event = datePrecision.adjustDateUpwards(event);

        return df.format(event);
    }

    public QueryPreprocessorResult preprocess(String segment, boolean appendFieldName) {
        QueryPreprocessorResult res = new QueryPreprocessorResult();
        res.setOriginalQuery(segment);
        Matcher centuryRangeMatcher = yearMonthPattern.matcher(segment);
        StringBuffer sb = new StringBuffer();

        String fieldStr = appendFieldName ? "date:" : "";

        while (centuryRangeMatcher.find()) {
            //-- not null
            String startStr = centuryRangeMatcher.group(1);
            //-- can be null
            String timelinePartSelector = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(3)).toLowerCase();
            //-- not null
            String yearStr = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(4)).toLowerCase();
            //-- can be null
            String monthStr = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(6)).toLowerCase();
            if ("".equals(monthStr))
                monthStr = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(7)).toLowerCase();

            boolean isIsz = true;
            if (!"".equals(timelinePartSelector)) {
                if (BEFORE_CHRISTMAS.equals(timelinePartSelector))
                    isIsz = false;
            }

            int year = 0;
            boolean validYear = true;
            try {
                year = Integer.parseInt(yearStr);

            } catch (NumberFormatException nex) {
                validYear = false;
            }

            String yearMonthQuery = startStr;
            if (validYear) {
                MonthContainer mc = monthConversionMap.get(monthStr);

                yearMonthQuery = startStr + fieldStr + "[" + formatStartMonth(year, mc) + " TO " + formatEndMonth(year, mc, isLeapYear(year)) + "]";
            }

            centuryRangeMatcher.appendReplacement(sb, yearMonthQuery);
            res.setQueryModified(true);
        }
        centuryRangeMatcher.appendTail(sb);

        res.setFilteredQuery(sb.toString());
        return res;
    }
}
