package hu.kozadat.search.interactive;

import hu.kozadat.search.Constants;
import hu.kozadat.search.exception.InteractiveSearchTermParseException;
import hu.scriptum.nda.api.MDcSearch;
import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class InteractiveSearchTermUtils {

    private static ObjectMapper mapper = new ObjectMapper();

    protected static String getStringValue(Map<String, Object> valueHolder, String key) {
        Object value = valueHolder.get(key);
        if (value instanceof String) {
            return (String) value;
        } else {
            return null;
        }
    }

    protected static List<String> getStringArrayValue(Map<String, Object> valueHolder, String key) {
        Object value = valueHolder.get(key);
        if (value instanceof List) {
            return (List) value;
        } else {
            return null;
        }
    }

    public static KeywordSearchTerm globalKeywordSearchTerm(Map<String, Object> term) {
        String expression = com.actiwise.einfoszab.Utils.null2String(getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_VALUE));
        String operator = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_OPERATOR);

        if (operator == null || !(Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_AND.equals(operator.toLowerCase()) || Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_OR.equals(operator.toLowerCase()))) {
            throw new IllegalArgumentException("globalKeywordSearchTermFromJSONObject(): operator cannot be empty");
        }

        KeywordSearchTerm keywordSearchTerm = new KeywordSearchTerm();
        keywordSearchTerm.setFieldName(null);
        keywordSearchTerm.setExpression(expression);
        keywordSearchTerm.setOperator(operator);

        return keywordSearchTerm;
    }

    public static KeywordSearchTerm keywordSearchTerm(Map<String, Object> term) {
        String field = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_FIELD);
        String expression = com.actiwise.einfoszab.Utils.null2String(getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_VALUE));
        String operator = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_OPERATOR);

        if (operator == null || !(Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_AND.equals(operator.toLowerCase()) || Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_OR.equals(operator.toLowerCase()))) {
            throw new IllegalArgumentException("keywordSearchTermFromJSONObject(): operator cannot be empty");
        }

        KeywordSearchTerm keywordSearchTerm = new KeywordSearchTerm();
        keywordSearchTerm.setFieldName(field);
        keywordSearchTerm.setExpression(expression);
        keywordSearchTerm.setOperator(operator);

        return keywordSearchTerm;
    }

    public static ValuedListSearchTerm valuedListSearchTerm(Map<String, Object> term) {
        String field = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_FIELD);
        List<String> values = getStringArrayValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_VALUE);
        if (values == null) {
            values = new ArrayList<String>();
        }
        String operator = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_OPERATOR);

        if (operator == null || !(Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_AND.equals(operator.toLowerCase()) || Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_OR.equals(operator.toLowerCase()))) {
            throw new IllegalArgumentException("valuedListSearchTermFromJSONObject: operator cannot be empty");
        }

        ValuedListSearchTerm valuedListSearchTerm = new ValuedListSearchTerm();
        valuedListSearchTerm.setFieldName(field);
        valuedListSearchTerm.setValues(values);
        valuedListSearchTerm.setOperator(operator);

        return valuedListSearchTerm;
    }

    public static NamespaceEntitySearchTerm nameSpaceSearchTerm(Map<String, Object> term) {
        String id = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_ID);
        String name = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_NAME);
        String operator = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_OPERATOR);

        if (operator == null || !(Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_AND.equals(operator.toLowerCase()) || Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_OR.equals(operator.toLowerCase()))) {
            throw new IllegalArgumentException("nameSpaceSearchTermFromJSONObject: operator cannot be empty");
        }

        if (id == null) {
            throw new IllegalArgumentException("nameSpaceSearchTermFromJSONObject: id cannot be empty");
        }
        if (name == null) {
            throw new IllegalArgumentException("nameSpaceSearchTermFromJSONObject: name cannot be empty");
        }

        NamespaceEntitySearchTerm namespaceEntitySearchTerm = new NamespaceEntitySearchTerm();
        namespaceEntitySearchTerm.setId(id);
        namespaceEntitySearchTerm.setName(name);
        namespaceEntitySearchTerm.setOperator(operator);

        return namespaceEntitySearchTerm;
    }

    public static IInteractiveSearchTerm fromJSON(Map<String, Object> term) {
        String type = getStringValue(term, Constants.INTERACTIVE_SEARCH_TERM_FIELD_TYPE);
        IInteractiveSearchTerm searchTerm;

        if (type == null || "".equals(type)) {
            throw new IllegalArgumentException("fromJSONObject(): type cannot be empty");
        }

        if (Constants.INTERACTIVE_SEARCH_TERM_NAME_GLOBAL.equals(type)) {
            searchTerm = globalKeywordSearchTerm(term);
        } else if (Constants.INTERACTIVE_SEARCH_TERM_NAME_FREETEXT.equals(type)) {
            searchTerm = keywordSearchTerm(term);
        } else if (Constants.INTERACTIVE_SEARCH_TERM_NAME_SIMPLELIST.equals(type)) {
            searchTerm = valuedListSearchTerm(term);
        } else if (Constants.INTERACTIVE_SEARCH_TERM_NAME_MULTILIST.equals(type)) {
            searchTerm = valuedListSearchTerm(term);
        } else if (Constants.INTERACTIVE_SEARCH_TERM_NAME_NAMESPACE.equals(type)) {
            searchTerm = nameSpaceSearchTerm(term);
        } else {
            throw new IllegalArgumentException("Cannot process JSON: " + term);
        }
        return searchTerm;
    }

    public static InteractiveSearchExpression fromJSONArray(List<Map<String, Object>> terms) {
        List<IInteractiveSearchTerm> searchTerms = new ArrayList<IInteractiveSearchTerm>();

        for (Map<String, Object> term : terms) {
            searchTerms.add(InteractiveSearchTermUtils.fromJSON(term));
        }

        return new InteractiveSearchExpression(searchTerms);
    }

    public static InteractiveSearchExpression fromJSONString(String jsonContent) throws InteractiveSearchTermParseException {

        try {
            List<Map<String, Object>> terms = mapper.readValue(jsonContent, new TypeReference<List<Map<String, Object>>>() {
            });
            return fromJSONArray(terms);
        } catch (JsonMappingException e) {
            throw new InteractiveSearchTermParseException(e.getMessage(), e);
        } catch (JsonParseException e) {
            throw new InteractiveSearchTermParseException(e.getMessage(), e);
        } catch (IOException e) {
            throw new InteractiveSearchTermParseException(e.getMessage(), e);
        }

    }

    public static IInteractiveSearchTerm getDefaultSearchTerm() {
        ValuedListSearchTerm res = new ValuedListSearchTerm();
        res.setFieldName(MDcSearch.FIELD_SCHEMA);
        res.setOperator(Constants.INTERACTIVE_SEARCH_TERM_OPERATOR_OR);

        List<String> schemaLst = new ArrayList<String>();
        schemaLst.add("Archív");
        schemaLst.add("Adatbázis");
        schemaLst.add("Általános");
        schemaLst.add("Közlöny");
        schemaLst.add("Jogszabálytervezet");
        schemaLst.add("Országgyűlési törvényjavaslat");
        res.setValues(schemaLst);

        return res;
    }
}
