package hu.kozadat.search.model;

import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.06.18.
 * Time: 15:32
 */
public class IndexerSearchResult extends FormChangeAwareSearchResult {

    private String sessionId;
    private String query;
    private Integer resultSize = 0;
    private Long executionTime = 0L;
    private Long processTime = 0L;
    private String charactersRemovedFromQuery;
    private List<IndexerSearchResultItem> items = new LinkedList<IndexerSearchResultItem>();
    private Map<Long, Collection<IndexerSearchResultItem>> itemsByDataProviderIds = new LinkedHashMap<Long, Collection<IndexerSearchResultItem>>();
    private Map<String, IndexerSearchResultItem> itemsByOaiRecordId = new LinkedHashMap<String, IndexerSearchResultItem>();
    private List<IndexerSearchResultItem> firstItems = new LinkedList<IndexerSearchResultItem>();

    public String getSessionId() {
        return sessionId;
    }

    public void setSessionId(String sessionId) {
        this.sessionId = sessionId;
    }

    public String getQuery() {
        return query;
    }

    public void setQuery(String query) {
        this.query = query;
    }

    public Integer getResultSize() {
        return resultSize;
    }

    public void setResultSize(Integer resultSize) {
        this.resultSize = resultSize;
    }

    public Long getExecutionTime() {
        return executionTime;
    }

    public void setExecutionTime(Long executionTime) {
        this.executionTime = executionTime;
    }

    public Long getProcessTime() {
        return processTime;
    }

    public void setProcessTime(Long processTime) {
        this.processTime = processTime;
    }

    public String getCharactersRemovedFromQuery() {
        return charactersRemovedFromQuery;
    }

    public void setCharactersRemovedFromQuery(String charactersRemovedFromQuery) {
        this.charactersRemovedFromQuery = charactersRemovedFromQuery;
    }

    public void addItem(IndexerSearchResultItem item) {
        if (item == null) {
            throw new IllegalArgumentException("item is null");
        }
        Long dataProviderId = item.getDataProviderId();
        if (dataProviderId == null) {
            throw new IllegalArgumentException("item.dataProviderId is null");
        }

        items.add(item);
        itemsByOaiRecordId.put(item.getRecordId(), item);

        Collection<IndexerSearchResultItem> itemsByDataProvider = itemsByDataProviderIds.get(dataProviderId);
        if (itemsByDataProvider == null) {
            itemsByDataProvider = new TreeSet<>(new IndexerSearchResultItemComparator());
            itemsByDataProviderIds.put(dataProviderId, itemsByDataProvider);
            firstItems.add(item);
        }

        itemsByDataProvider.add(item);

    }

    public List<IndexerSearchResultItem> getItems() {
        return items;
    }

    public Map<Long, Collection<IndexerSearchResultItem>> getItemsByDataProviderIds() {
        return itemsByDataProviderIds;
    }

    public List<IndexerSearchResultItem> getFirstItems() {
        return firstItems;
    }

    public Map<String, IndexerSearchResultItem> getItemsByOaiRecordId() {
        return itemsByOaiRecordId;
    }

    public IndexerSearchResult getItemsByDataProviderAsSearchResult(Long dataProviderId) {
        if (itemsByDataProviderIds != null && dataProviderId != null && itemsByDataProviderIds.containsKey(dataProviderId)) {

            IndexerSearchResult resultPart = new IndexerSearchResult();

            resultPart.setExecutionTime(this.getExecutionTime());
            resultPart.setProcessTime(this.getProcessTime());
            resultPart.setQuery(this.getQuery());
            resultPart.setSessionId(this.getSessionId());
            resultPart.setCharactersRemovedFromQuery(this.getCharactersRemovedFromQuery());

            Collection<IndexerSearchResultItem> resultPartItems = itemsByDataProviderIds.get(dataProviderId);
            if (resultPartItems != null && !resultPartItems.isEmpty()) {
                resultPart.setResultSize(resultPartItems.size());
                for (IndexerSearchResultItem item : resultPartItems) {
                    resultPart.addItem(item);
                }
                return resultPart;
            }

        }
        return null;
    }
}
