package hu.kozadat.search.model;

import java.io.Serializable;
import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.06.18.
 * Time: 15:38
 */
public class IndexerSearchResultItem  implements Serializable {

    private static final String EMPTY_QUALIFIER = "";
    private String recordId;
    private String dataProviderName;
    private Long dataProviderId;
    private String schemaName;
    private IndexerSearchResultSchemaType schemaType;
    private Date modificationDate;
    private boolean archiveWebpage = false;
    private String attachmentViewerUrl;

    private Set<String> keywords = new LinkedHashSet<String>();

    private final Map<String, Map<String, List<String>>> schemaElements = new LinkedHashMap<String, Map<String, List<String>>>();

    public String getRecordId() {
        return recordId;
    }

    public void setRecordId(String recordId) {
        this.recordId = recordId;
    }

    public String getDataProviderName() {
        return dataProviderName;
    }

    public void setDataProviderName(String dataProviderName) {
        this.dataProviderName = dataProviderName;
    }

    public Long getDataProviderId() {
        return dataProviderId;
    }

    public void setDataProviderId(Long dataProviderId) {
        this.dataProviderId = dataProviderId;
    }

    public String getSchemaName() {
        return schemaName;
    }

    public void setSchemaName(String schemaName) {
        this.schemaName = schemaName;
    }

    public IndexerSearchResultSchemaType getSchemaType() {
        return schemaType;
    }

    public void setSchemaType(IndexerSearchResultSchemaType schemaType) {
        this.schemaType = schemaType;
    }

    public Date getModificationDate() {
        return modificationDate;
    }

    public void setModificationDate(Date modificationDate) {
        this.modificationDate = modificationDate;
    }

    public Set<String> getKeywords() {
        return keywords;
    }

    public void addKeyword(String keyword) {
        this.keywords.add(keyword);
    }

    public String getKeywordString() {
        StringBuilder keywordStringBuilder = new StringBuilder();
        if (keywords != null && !keywords.isEmpty()) {
            boolean first = true;
            for (String keyword : keywords) {
                if (!first) {
                    keywordStringBuilder.append(", ");
                }
                if (first) {
                    first = false;
                }
                keywordStringBuilder.append(keyword);
            }
        }
        return keywordStringBuilder.toString();
    }

    public Map<String, Map<String, List<String>>> getSchemaElements() {
        return schemaElements;
    }

    public void addSchemaElement(String schemaElementName, String schemaElementQualifier, String value) {
        if (schemaElementName == null) {
            throw new IllegalArgumentException("schemaElementName cannot be null");
        }
        Map<String, List<String>> schemaElementQualifiers = schemaElements.get(schemaElementName);
        if (schemaElementQualifiers == null) {
            schemaElementQualifiers = new LinkedHashMap<String, List<String>>();
            schemaElements.put(schemaElementName, schemaElementQualifiers);
        }
        List<String> schemaQualifierValues = schemaElementQualifiers.get(schemaElementQualifier);
        if (schemaQualifierValues == null) {
            schemaQualifierValues = new LinkedList<String>();
            if (schemaElementQualifier == null) {
                schemaElementQualifiers.put(EMPTY_QUALIFIER, schemaQualifierValues);
            } else {
                schemaElementQualifiers.put(schemaElementQualifier, schemaQualifierValues);
            }
        }
        schemaQualifierValues.add(value);
    }

    public List<String> getSchemaElementValue(String schemaElementName, String schemaElementQualifier) {
        if (schemaElementName == null) {
            throw new IllegalArgumentException("schemaElementName cannot be null");
        }
        Map<String, List<String>> schemaElementQualifiers = schemaElements.get(schemaElementName);
        if (schemaElementQualifiers == null) {
            return null;
        }
        return schemaElementQualifiers.get(schemaElementQualifier);
    }

    public boolean isArchiveWebpage() {
        return archiveWebpage;
    }

    public void setArchiveWebpage(boolean archiveWebpage) {
        this.archiveWebpage = archiveWebpage;
    }

    public String getAttachmentViewerUrl() {
        return attachmentViewerUrl;
    }

    public void setAttachmentViewerUrl(String attachmentViewerUrl) {
        this.attachmentViewerUrl = attachmentViewerUrl;
    }

    @Override
    public String toString() {
        return "Item{" +
                recordId +
                "; " + schemaType +
                "; " + dataProviderId +
                '}';
    }

}
