package hu.kozadat.search.persistence;

import com.actiwise.einfoszab.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import java.util.Enumeration;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

@Repository
public class LabelCache implements InitializingBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(LabelCache.class);

    private Map<String, String> elementLabels = new TreeMap<String, String>();
    private DublinCoreResourceBundleMessageSource dublinCoreElements;

    @Autowired
    @Qualifier("dublinCoreElements")
    public void setDublinCoreElements(DublinCoreResourceBundleMessageSource dublinCoreElements) {
        this.dublinCoreElements = dublinCoreElements;
    }

    public void afterPropertiesSet() throws Exception {
        Enumeration<String> propertyNames = dublinCoreElements.getPropertyNames();
        while (propertyNames.hasMoreElements()) {
            String propertyName = propertyNames.nextElement();
            String label = dublinCoreElements.getMessage(propertyName, new Object[]{}, Locale.getDefault());
            String originalLabel = null;
            if (propertyName.startsWith(SearchCache.ELEMENT_PREFIX)) {
                originalLabel = propertyName.replace(SearchCache.ELEMENT_PREFIX, "");
            } else if (propertyName.startsWith(SearchCache.QUALIFIER_PREFIX)) {
                originalLabel = propertyName.replace(SearchCache.QUALIFIER_PREFIX, "");
            }
            if (originalLabel != null) {
                elementLabels.put(Utils.normalizeFieldName(label), originalLabel);
            } else {
                LOGGER.warn("LabelCache malfunction!");
            }
        }
    }

    public String getLabelTranslation(String label) {
        return elementLabels.get(label);
    }

}
