package hu.kozadat.search.persistence.impl;

import hu.kozadat.search.model.DataProviderDisplayInfo;
import hu.kozadat.search.persistence.DataProviderDisplayInfoOperations;
import hu.kozadat.search.persistence.core.DataProviderDisplayInfoDAO;
import hu.scriptum.einfoszab.datamodel.Address;
import hu.scriptum.einfoszab.datamodel.DataProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.05.24.
 * Time: 11:55
 * To change this template use File | Settings | File Templates.
 */
@Repository("dataProviderDisplayInfoOperations")
public class DataProviderDisplayInfoOperationsImpl implements DataProviderDisplayInfoOperations {

    private static final Logger LOGGER = LoggerFactory.getLogger(DataProviderDisplayInfoOperationsImpl.class);
    private static final String POSTFIX_ARCHI = " - ARCH\u00CDV";

    private JdbcTemplate jdbcTemplate;
    private DataProviderDisplayInfoDAO dataProviderDisplayInfoDAO = new DataProviderDisplayInfoDAO();

    @Autowired
    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public DataProviderDisplayInfo fromName(String name, String type) {
        return dataProviderDisplayInfoDAO.fromName(jdbcTemplate, name, type);
    }

    public Address getAddress(long dataProviderId) {
        return dataProviderDisplayInfoDAO.getAddress(jdbcTemplate, dataProviderId);
    }

    public DataProviderDisplayInfo fromId(int id) {
        DataProviderDisplayInfo dataProviderDisplayInfo = dataProviderDisplayInfoDAO.fromId(jdbcTemplate, id);
        updateInstitutionName(dataProviderDisplayInfo);
        return dataProviderDisplayInfo;
    }

    public DataProviderDisplayInfo fromId(long id, String type) {
        DataProviderDisplayInfo dataProviderDisplayInfo = dataProviderDisplayInfoDAO.fromId(jdbcTemplate, id, type);
        updateInstitutionName(dataProviderDisplayInfo);
        return dataProviderDisplayInfo;
    }

    private void updateInstitutionName(DataProviderDisplayInfo dataProviderDisplayInfo) {
        if (DataProvider.STATE_ARCHIVED.equals(dataProviderDisplayInfo.getState())) {
            dataProviderDisplayInfo.setInstitutionName(dataProviderDisplayInfo.getInstitutionName() + POSTFIX_ARCHI);
        }
    }

}
