package hu.kozadat.search.persistence.impl;

import hu.kozadat.search.exception.UserOperationException;
import hu.kozadat.search.persistence.UserOperations;
import hu.scriptum.einfoszab.api.EInfoSzabOperationException;
import hu.scriptum.einfoszab.datamodel.Contact;
import hu.scriptum.einfoszab.datamodel.ContactInfo;
import hu.scriptum.einfoszab.datamodel.Person;
import hu.scriptum.einfoszab.rdbms.RdbmsUserOperations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.05.23.
 * Time: 9:30
 * To change this template use File | Settings | File Templates.
 */
@Repository("userOperations")
public class UserOperationsImpl implements UserOperations {

    private JdbcTemplate jdbcTemplate;

    private RdbmsUserOperations rdbmsUserOperations;

    @Autowired
    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.rdbmsUserOperations = new RdbmsUserOperations();
        this.rdbmsUserOperations.setJdbcTemplate(this.jdbcTemplate);
    }

    @Deprecated
    public RdbmsUserOperations getRdbmsUserOperations() {
        return rdbmsUserOperations;
    }

    public synchronized Person getLoggedPerson(String loginName) throws UserOperationException {
        try {
            return rdbmsUserOperations.getLoggedPerson(loginName);
        } catch (EInfoSzabOperationException e) {
            throw new UserOperationException(e.getMessage(), e);
        }
    }

    public synchronized void deletePerson(Person person) throws UserOperationException {
        try {
            rdbmsUserOperations.deletePerson(person);
        } catch (EInfoSzabOperationException e) {
            throw new UserOperationException(e.getMessage(), e);
        }
    }

    public Map<String, Integer> deletePersonInternal(String loginName) {
        return rdbmsUserOperations.deletePersonInternal(loginName);
    }

    public synchronized void savePerson(Person person) throws UserOperationException {
        try {
            rdbmsUserOperations.savePerson(person);
        } catch (EInfoSzabOperationException e) {
            throw new UserOperationException(e.getMessage(), e);
        }
    }

    public void savePersonInternal(Person person) {
        rdbmsUserOperations.savePersonInternal(person);
    }

    public synchronized void createPerson(Person person) throws UserOperationException {
        try {
            rdbmsUserOperations.createPerson(person);
        } catch (EInfoSzabOperationException e) {
            throw new UserOperationException(e.getMessage(), e);
        }
    }

    public void createPersonInternal(Person person) throws UserOperationException {
        try {
            rdbmsUserOperations.createPersonInternal(person);
        } catch (EInfoSzabOperationException e) {
            throw new UserOperationException(e.getMessage(), e);
        }
    }

    public void changePassword(String loginName, String oldPassword, String newPassword, String changeRole) throws UserOperationException {
        try {
            rdbmsUserOperations.changePassword(loginName, oldPassword, newPassword, changeRole);
        } catch (EInfoSzabOperationException e) {
            throw new UserOperationException(e.getMessage(), e);
        }
    }

    public List getPersons(String userGroupName) {
        return rdbmsUserOperations.getPersons(userGroupName);
    }

    public void addUserPhones(Person p) {
        rdbmsUserOperations.addUserPhones(p);
    }

    public int deleteUserPhones(String loginName) {
        return rdbmsUserOperations.deleteUserPhones(loginName);
    }

    public void addUserEmails(Person p) {
        rdbmsUserOperations.addUserEmails(p);
    }

    public Map<String, Integer> deleteResponsible(Person responsible) {
        return rdbmsUserOperations.deleteResponsible(responsible);
    }

    public String promoteAContact(long dataProviderId) {
        return rdbmsUserOperations.promoteAContact(dataProviderId);
    }

    public int deleteUserEmails(String loginName) {
        return rdbmsUserOperations.deleteUserEmails(loginName);
    }

    public List<ContactInfo> getContactInfos(String dataProviderName) {
        return rdbmsUserOperations.getContactInfos(dataProviderName);
    }

    public Person getPersonByLoginName(String loginName) {
        return rdbmsUserOperations.getPersonByLoginName(loginName);
    }

    public Contact getContactByLoginName(String loginName) {
        return rdbmsUserOperations.getContactByLoginName(loginName);
    }

    public List<Person> getContacts(long dataProviderId, String role) {
        return rdbmsUserOperations.getContacts(dataProviderId, role);
    }

    public int updatePerson(Person p) {
        return rdbmsUserOperations.updatePerson(p);
    }

    public int updateContact(Contact c) {
        return rdbmsUserOperations.updateContact(c);
    }

    public boolean existsWithLoginName(String loginName) {
        return rdbmsUserOperations.existsWithLoginName(loginName);
    }

    public String toString() {
        return rdbmsUserOperations.toString();
    }
}
