package hu.kozadat.search.persistence.institutionsearch;

import hu.kozadat.search.model.InstitutionSearchResult;
import hu.kozadat.search.model.InstitutionSearchResultItem;
import hu.kozadat.search.model.InstitutionSearchResultItemType;
import hu.scriptum.einfoszab.datamodel.DataProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcOperations;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * Intezmenyi keresest vegzo programkod.
 *
 * @author Csaba Horvath, Scriptum Inc.
 * @version $Id$
 */
@Repository("institutionSearch")
public class InstitutionSearch {

    private JdbcOperations jdbcTemplate;

    private static final Logger LOGGER = LoggerFactory.getLogger(InstitutionSearch.class);

    private static final String TABLE_EXTERNAL_INSTITUTE = "external_institute";
    private static final String COLUMN_EXTERNAL_INSTITUTE_ID = "id";
    private static final String COLUMN_EXTERNAL_INSTITUTE_NAME = "name";

    private static final String TABLE_DATA_PROVIDERS = "data_providers";
    private static final String COLUMN_DATA_PROVIDERS_NAME = "name";
    private static final String COLUMN_DATA_PROVIDERS_ID = "id";
    private static final String COLUMN_DATA_PROVIDERS_STATE = "state";

    private static final String LOWER_DATA_PROVIDER_NAME_LIKE = "LOWER(" + COLUMN_DATA_PROVIDERS_NAME + ") LIKE ?";
    private static final String LOWER_EXTERNAL_INSTITUTE_NAME_LIKE = "LOWER(" + COLUMN_EXTERNAL_INSTITUTE_NAME + ") LIKE ?";

    private static final String OR_STATEMENT = " OR ";

    private static final String QUERY_DATA_PROVIDERS =
            "SELECT "
                    + COLUMN_DATA_PROVIDERS_ID + ", "
                    + COLUMN_DATA_PROVIDERS_NAME + ", "
                    + COLUMN_DATA_PROVIDERS_STATE
                    + " FROM "
                    + TABLE_DATA_PROVIDERS
                    + " WHERE "
                    + COLUMN_DATA_PROVIDERS_STATE + " IN (" + "'A'" + "," + "'R'" + "," + "'I" + ")"
                    + " AND "
                    + COLUMN_DATA_PROVIDERS_NAME + " LIKE " + "?"
                    + " ORDER BY "
                    + COLUMN_DATA_PROVIDERS_NAME;

    private static final String QUERY_DATA_PROVIDERS_CASE_INSENSITIVE =
            "SELECT "
                    + COLUMN_DATA_PROVIDERS_ID + ", "
                    + COLUMN_DATA_PROVIDERS_NAME + ", "
                    + COLUMN_DATA_PROVIDERS_STATE
                    + " FROM "
                    + TABLE_DATA_PROVIDERS
                    + " WHERE "
                    + COLUMN_DATA_PROVIDERS_STATE + " IN (" + "'A'" + "," + "'R'" + "," + "'I'" + ")"
                    + " AND "
                    + LOWER_DATA_PROVIDER_NAME_LIKE
                    + " ORDER BY "
                    + COLUMN_DATA_PROVIDERS_NAME;

    private static final String QUERY_EXTERNAL_INSTITUTES =
            "SELECT "
                    + COLUMN_EXTERNAL_INSTITUTE_ID + ", "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME
                    + " FROM "
                    + TABLE_EXTERNAL_INSTITUTE
                    + " WHERE "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME + " LIKE " + "?"
                    + " ORDER BY "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME;

    private static final String QUERY_EXTERNAL_INSTITUTES_CASE_INSENSITIVE =
            "SELECT "
                    + COLUMN_EXTERNAL_INSTITUTE_ID + ", "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME
                    + " FROM "
                    + TABLE_EXTERNAL_INSTITUTE
                    + " WHERE "
                    + LOWER_EXTERNAL_INSTITUTE_NAME_LIKE
                    + " ORDER BY "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME;

    private static final String QUERY_DATA_PROVIDERS_BY_LETTER_HEAD =
            "SELECT "
                    + COLUMN_DATA_PROVIDERS_ID + ", "
                    + COLUMN_DATA_PROVIDERS_NAME + ", "
                    + COLUMN_DATA_PROVIDERS_STATE
                    + " FROM "
                    + TABLE_DATA_PROVIDERS
                    + " WHERE "
                    + COLUMN_DATA_PROVIDERS_STATE + " IN " + "(" + "'A'" + "," + "'R'" + "," + "'I'" + ") "
                    + " AND (";

    private static final String QUERY_DATA_PROVIDERS_BY_LETTER_TAIL =
            ")"
                    + " ORDER BY "
                    + COLUMN_DATA_PROVIDERS_NAME;

    private static final String QUERY_EXTERNAL_INSTITUTES_BY_LETTER_HEAD =
            "SELECT "
                    + COLUMN_EXTERNAL_INSTITUTE_ID + ", "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME
                    + " FROM "
                    + TABLE_EXTERNAL_INSTITUTE
                    + " WHERE "
                    + " (";

    private static final String QUERY_EXTERNAL_INSTITUTES_BY_LETTER_TAIL =
            ")"
                    + " ORDER BY "
                    + COLUMN_EXTERNAL_INSTITUTE_NAME;

    @Deprecated
    public List<InstitutionSearchResultItem> search(String institutionNamePattern, final int fromIndex, final int numOfHits) {
        List<InstitutionSearchResultItem> aggregatedResult = new LinkedList<InstitutionSearchResultItem>();

        String caseInsensitivePattern = (institutionNamePattern != null ? institutionNamePattern.toLowerCase() : "");

        List<InstitutionSearchResultItem> dataProviders = jdbcTemplate.query(QUERY_DATA_PROVIDERS_CASE_INSENSITIVE,
                new Object[]{"%" + caseInsensitivePattern + "%"},
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setDataproviderId(resultSet.getLong(COLUMN_DATA_PROVIDERS_ID));
                        institution.setInstitutionName(resultSet.getString(COLUMN_DATA_PROVIDERS_NAME));
                        String state = resultSet.getString(COLUMN_DATA_PROVIDERS_STATE);
                        guessInstituteType(state, institution);
                        return institution;
                    }
                });

        List<InstitutionSearchResultItem> externalInstitutes = jdbcTemplate.query(QUERY_EXTERNAL_INSTITUTES_CASE_INSENSITIVE,
                new Object[]{"%" + caseInsensitivePattern + "%"},
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setDataproviderId(resultSet.getLong(COLUMN_EXTERNAL_INSTITUTE_ID));
                        institution.setInstitutionName(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_NAME));
                        institution.setType(InstitutionSearchResultItemType.ExternalInstitute);
                        return institution;
                    }
                });

        for (InstitutionSearchResultItem entry : dataProviders) {
            if (entry != null) {
                aggregatedResult.add(entry);
            }
        }
        for (InstitutionSearchResultItem entry : externalInstitutes) {
            if (entry != null) {
                aggregatedResult.add(entry);
            }
        }
        Collections.sort(aggregatedResult, new InstitutionSearchResultItemNameComparator());

        return aggregatedResult;
    }

    public InstitutionSearchResult search(String institutionNamePattern) {
        InstitutionSearchResult institutionSearchResult = new InstitutionSearchResult();

        String caseInsensitivePattern = (institutionNamePattern != null ? institutionNamePattern.toLowerCase() : "");

        List<InstitutionSearchResultItem> dataProviders = jdbcTemplate.query(QUERY_DATA_PROVIDERS_CASE_INSENSITIVE,
                new Object[]{"%" + caseInsensitivePattern + "%"},
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setDataproviderId(resultSet.getLong(COLUMN_DATA_PROVIDERS_ID));
                        institution.setInstitutionName(resultSet.getString(COLUMN_DATA_PROVIDERS_NAME));
                        String state = resultSet.getString(COLUMN_DATA_PROVIDERS_STATE);
                        guessInstituteType(state, institution);
                        return institution;
                    }
                });

        List<InstitutionSearchResultItem> externalInstitutes = jdbcTemplate.query(QUERY_EXTERNAL_INSTITUTES_CASE_INSENSITIVE,
                new Object[]{"%" + caseInsensitivePattern + "%"},
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setDataproviderId(resultSet.getLong(COLUMN_EXTERNAL_INSTITUTE_ID));
                        institution.setInstitutionName(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_NAME));
                        institution.setType(InstitutionSearchResultItemType.ExternalInstitute);
                        return institution;
                    }
                });

        for (InstitutionSearchResultItem item : dataProviders) {
            if (item != null) {
                institutionSearchResult.getItems().add(item);
            }
        }
        for (InstitutionSearchResultItem item : externalInstitutes) {
            if (item != null) {
                institutionSearchResult.getItems().add(item);
            }
        }
        Collections.sort(institutionSearchResult.getItems(), new InstitutionSearchResultItemNameComparator());

        return institutionSearchResult;
    }

    private void guessInstituteType(String state, InstitutionSearchResultItem institution) {
        if (DataProvider.STATE_ACTIVE.equals(state)) {
            institution.setType(InstitutionSearchResultItemType.ActiveDataProvider);
        } else if (DataProvider.STATE_REGISTERED.equals(state)) {
            institution.setType(InstitutionSearchResultItemType.RegisteredDataProvider);
        } else if (DataProvider.STATE_ARCHIVED.equals(state)) {
            institution.setType(InstitutionSearchResultItemType.ArchivedDataProvider);
            // tnagy mantis issue fix 0001448
            institution.setInstitutionName(institution.getInstitutionName() + " - ARCH\u00CDV");
        }
    }

    @Deprecated
    public List<InstitutionSearchResultItem> searchByLeadingLetters(String[] leadingLetters, final int fromIndex, final int numOfHits) {

        List<InstitutionSearchResultItem> aggregatedResult = new LinkedList<InstitutionSearchResultItem>();

        // tnagy query data providers
        StringBuilder queryDataProviders = new StringBuilder(QUERY_DATA_PROVIDERS_BY_LETTER_HEAD);
        List<String> dataProviderParams = new ArrayList<String>(leadingLetters.length);
        for (int i = 0; i < leadingLetters.length; i++) {
            queryDataProviders.append(LOWER_DATA_PROVIDER_NAME_LIKE);
            dataProviderParams.add(leadingLetters[i] + "%");
            if (i != leadingLetters.length - 1) {
                queryDataProviders.append(OR_STATEMENT);
            }
        }
        queryDataProviders.append(QUERY_DATA_PROVIDERS_BY_LETTER_TAIL);

        List<InstitutionSearchResultItem> dataProviders = jdbcTemplate.query(queryDataProviders.toString(),
                dataProviderParams.toArray(),
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setInstitutionName(resultSet.getString(COLUMN_DATA_PROVIDERS_NAME));
                        institution.setDataproviderId(resultSet.getLong(COLUMN_DATA_PROVIDERS_ID));
                        String state = resultSet.getString(COLUMN_DATA_PROVIDERS_STATE);
                        guessInstituteType(state, institution);
                        return institution;
                    }
                });

        // tnagy query external institutes
        StringBuilder queryExternalInstitutes = new StringBuilder(QUERY_EXTERNAL_INSTITUTES_BY_LETTER_HEAD);
        List<String> externalInstitutesParams = new ArrayList<String>(leadingLetters.length);
        for (int i = 0; i < leadingLetters.length; i++) {
            queryExternalInstitutes.append(LOWER_EXTERNAL_INSTITUTE_NAME_LIKE);
            externalInstitutesParams.add(leadingLetters[i] + "%");
            if (i != leadingLetters.length - 1) {
                queryExternalInstitutes.append(OR_STATEMENT);
            }
        }
        queryExternalInstitutes.append(QUERY_EXTERNAL_INSTITUTES_BY_LETTER_TAIL);

        List<InstitutionSearchResultItem> externalInstitutes = jdbcTemplate.query(queryExternalInstitutes.toString(),
                externalInstitutesParams.toArray(),
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setInstitutionName(resultSet.getString(COLUMN_DATA_PROVIDERS_NAME));
                        institution.setDataproviderId(resultSet.getLong(COLUMN_DATA_PROVIDERS_ID));
                        institution.setType(InstitutionSearchResultItemType.ExternalInstitute);
                        return institution;
                    }
                });

        // tnay aggregate results
        for (InstitutionSearchResultItem entry : dataProviders) {
            if (entry != null) {
                aggregatedResult.add(entry);
            }
        }
        for (InstitutionSearchResultItem entry : externalInstitutes) {
            if (entry != null) {
                aggregatedResult.add(entry);
            }
        }
        Collections.sort(aggregatedResult, new InstitutionSearchResultItemNameComparator());

        return aggregatedResult;
    }

    public InstitutionSearchResult searchByLeadingLetters(String[] leadingLetters) {

        InstitutionSearchResult institutionSearchResult = new InstitutionSearchResult();

        // tnagy query data providers
        StringBuilder queryDataProviders = new StringBuilder(QUERY_DATA_PROVIDERS_BY_LETTER_HEAD);
        List<String> dataProviderParams = new ArrayList<String>(leadingLetters.length);
        for (int i = 0; i < leadingLetters.length; i++) {
            queryDataProviders.append(LOWER_DATA_PROVIDER_NAME_LIKE);
            dataProviderParams.add(leadingLetters[i] + "%");
            if (i != leadingLetters.length - 1) {
                queryDataProviders.append(OR_STATEMENT);
            }
        }
        queryDataProviders.append(QUERY_DATA_PROVIDERS_BY_LETTER_TAIL);

        List<InstitutionSearchResultItem> dataProviders = jdbcTemplate.query(queryDataProviders.toString(),
                dataProviderParams.toArray(),
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setInstitutionName(resultSet.getString(COLUMN_DATA_PROVIDERS_NAME));
                        institution.setDataproviderId(resultSet.getLong(COLUMN_DATA_PROVIDERS_ID));
                        String state = resultSet.getString(COLUMN_DATA_PROVIDERS_STATE);
                        guessInstituteType(state, institution);
                        return institution;
                    }
                });

        // tnagy query external institutes
        StringBuilder queryExternalInstitutes = new StringBuilder(QUERY_EXTERNAL_INSTITUTES_BY_LETTER_HEAD);
        List<String> externalInstitutesParams = new ArrayList<String>(leadingLetters.length);
        for (int i = 0; i < leadingLetters.length; i++) {
            queryExternalInstitutes.append(LOWER_EXTERNAL_INSTITUTE_NAME_LIKE);
            externalInstitutesParams.add(leadingLetters[i] + "%");
            if (i != leadingLetters.length - 1) {
                queryExternalInstitutes.append(OR_STATEMENT);
            }
        }
        queryExternalInstitutes.append(QUERY_EXTERNAL_INSTITUTES_BY_LETTER_TAIL);

        List<InstitutionSearchResultItem> externalInstitutes = jdbcTemplate.query(queryExternalInstitutes.toString(),
                externalInstitutesParams.toArray(),
                new RowMapper() {
                    public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                        InstitutionSearchResultItem institution = new InstitutionSearchResultItem();
                        institution.setInstitutionName(resultSet.getString(COLUMN_DATA_PROVIDERS_NAME));
                        institution.setDataproviderId(resultSet.getLong(COLUMN_DATA_PROVIDERS_ID));
                        institution.setType(InstitutionSearchResultItemType.ExternalInstitute);
                        return institution;
                    }
                });

        // tnay aggregate results
        for (InstitutionSearchResultItem item : dataProviders) {
            if (item != null) {
                institutionSearchResult.getItems().add(item);
            }
        }
        for (InstitutionSearchResultItem item : externalInstitutes) {
            if (item != null) {
                institutionSearchResult.getItems().add(item);
            }
        }
        Collections.sort(institutionSearchResult.getItems(), new InstitutionSearchResultItemNameComparator());

        return institutionSearchResult;
    }

    @Autowired
    public void setJdbcTemplate(JdbcOperations jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }
}
