package hu.kozadat.search.presentation;

import hu.kozadat.search.model.IndexerSearchResult;
import hu.kozadat.search.model.IndexerSearchResultItem;
import hu.kozadat.search.service.simplesearch.SimpleSearch;
import hu.kozadat.search.service.log.SearchLogger;
import hu.scriptum.nda.api.MDcSearch;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.LinkedList;

@Controller
public class RecordDetailsController {

    private static final Logger LOGGER = LoggerFactory.getLogger(RecordDetailsController.class);

    private SearchLogger searchLogger;
    private SimpleSearch simpleSearch;

    @Autowired
    public void setSearchLogger(SearchLogger searchLogger) {
        this.searchLogger = searchLogger;
    }

    @Autowired
    public void setSimpleSearch(SimpleSearch simpleSearch) {
        this.simpleSearch = simpleSearch;
    }

    @RequestMapping(value = "/rekord/{recordId}")
    public ModelAndView showRecordDetails(HttpServletRequest httpServletRequest, @PathVariable("recordId") String recordId, @RequestParam(value= "referer", required = false) String referer) throws Exception {

        IndexerSearchResult indexerSearchResult = (IndexerSearchResult) httpServletRequest.getSession().getAttribute("indexerSearchResult");
        IndexerSearchResultItem indexerSearchResultItem = null;

        if (indexerSearchResult != null) {
            indexerSearchResultItem = indexerSearchResult.getItemsByOaiRecordId().get(recordId);
        }
        if (indexerSearchResultItem == null) {

            StringBuilder searchExpressionBuilder = new StringBuilder();

            searchExpressionBuilder.append(MDcSearch.FIELD_RECORD_ID).append(":");
            searchExpressionBuilder.append(MDcSearch.ID_PREFIX).append(recordId).append(MDcSearch.ID_POSTFIX);

            String searchExpression = searchExpressionBuilder.toString();

            String sessionId = httpServletRequest.getSession().getId();
            indexerSearchResult = simpleSearch.search(searchExpression, new LinkedList<String>(), sessionId);
            int resultSize = indexerSearchResult.getResultSize();
            if (resultSize > 0) {
                simpleSearch.fillResult(indexerSearchResult, 0, resultSize);
                simpleSearch.close(sessionId);
            }

            indexerSearchResultItem = indexerSearchResult.getItemsByOaiRecordId().get(recordId);
        }

        String viewName = "recordDetails";
        ModelAndView modelAndView = new ModelAndView(viewName);
        modelAndView.addObject("searchResultItem", indexerSearchResultItem);


        if (referer == null || "".equals(referer.trim())) {
            modelAndView.addObject("referer", getRefererHeaderParam(httpServletRequest));
        } else {
            if (referer.startsWith("/")) {
                modelAndView.addObject("referer", referer);
            }
        }

        return modelAndView;

    }

    private String getRefererHeaderParam(HttpServletRequest request) {

        String referer = request.getHeader("Referer");

        if (referer == null) {
            return null;
        }

        return referer;
    }
}
