package hu.kozadat.search.presentation.institutionsearch;

import hu.kozadat.search.model.InstitutionNameAutoCompleteData;
import hu.kozadat.search.model.InstitutionNameAutoCompleteDataStatusType;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.service.institutionsearch.InstitutionNameAutoCompleteService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.06.29.
 * Time: 13:21
 * To change this template use File | Settings | File Templates.
 */
@Controller
public class InstitutionNameAutoCompleteController {

    private static final Logger LOGGER = LoggerFactory.getLogger(InstitutionNameAutoCompleteController.class);

    private InstitutionNameAutoCompleteService institutionNameAutoCompleteService;
    private SearchUIConfiguration searchUIConfiguration;

    @RequestMapping(value = "/intezmeny/kiegeszites.json", method = RequestMethod.GET)
    public
    @ResponseBody
    List<InstitutionNameAutoCompleteData> completeInstitutionName(@RequestParam(value = "term", required = false) String term) {

        List<InstitutionNameAutoCompleteData> autoCompleteDataList = new LinkedList<InstitutionNameAutoCompleteData>();

        // tnagy read matching institutes
        Set<InstitutionNameAutoCompleteData> byPrefix = institutionNameAutoCompleteService.findByPrefix(term);
        for (InstitutionNameAutoCompleteData data : byPrefix) {
            String instituteDetailsUrl;
            if (InstitutionNameAutoCompleteDataStatusType.ACTIVE.getJsonValue().equals(data.getStatus())
                    || InstitutionNameAutoCompleteDataStatusType.REGISTERED.getJsonValue().equals(data.getStatus())
                    || InstitutionNameAutoCompleteDataStatusType.ARCHIVED.getJsonValue().equals(data.getStatus())
                    ) {
                instituteDetailsUrl = searchUIConfiguration.getApplicationContext() + "/intezmeny/adatlap/" + data.getId();
            } else if (InstitutionNameAutoCompleteDataStatusType.OTHER.getJsonValue().equals(data.getStatus())) {
                instituteDetailsUrl = searchUIConfiguration.getApplicationContext() + "/intezmeny/egyeb/adatlap/" + data.getId();
            } else {
                LOGGER.warn("completeInstitutionName(): Unknown status type '" + data.getStatus() + "' for InstitutionNameAutoCompleteData.status");
                instituteDetailsUrl = null;
            }
            data.setValue(instituteDetailsUrl);
            // tnagy mantis issue fix 0001448
            if (InstitutionNameAutoCompleteDataStatusType.ARCHIVED.getJsonValue().equals(data.getStatus())) {
                data.setLabel(data.getLabel() + " - ARCH\u00CDV");
            }
        }
        autoCompleteDataList.addAll(byPrefix);

        // calculate more-labeled list item
        int resultSize = institutionNameAutoCompleteService.countByPrefix(term);
        if (autoCompleteDataList.size() < resultSize) {
            int more = resultSize - autoCompleteDataList.size();
            String moreUrl = searchUIConfiguration.getApplicationContext() + "/intezmeny/talalatok?nev=" + term;
            autoCompleteDataList.add(getMoreLabeledData(more, moreUrl));
        }

        return autoCompleteDataList;

    }

    private List<InstitutionNameAutoCompleteData> getDummyList() {
        List<InstitutionNameAutoCompleteData> autoCompleteDataList = new LinkedList<InstitutionNameAutoCompleteData>();

        autoCompleteDataList.add(new InstitutionNameAutoCompleteData(95L, "Dióskál, Egeraracsa", "/url", "other"));
        autoCompleteDataList.add(new InstitutionNameAutoCompleteData(99L, "Kálmáncsa Község Önkormányzata", "/url", "active"));
        autoCompleteDataList.add(new InstitutionNameAutoCompleteData(99L, "Káld Község Önkormányzata ", "/url", "registred"));
        autoCompleteDataList.add(new InstitutionNameAutoCompleteData(99L, "KÁLLÓSEMJÉN NAGYKÖZSÉG ", "/url", "active"));
        autoCompleteDataList.add(new InstitutionNameAutoCompleteData(99L, "...és még 156 további intézmény", "/all", "more"));

        return autoCompleteDataList;
    }

    private InstitutionNameAutoCompleteData getMoreLabeledData(int more, String moreUrl) {
        InstitutionNameAutoCompleteData moreLabeled = new InstitutionNameAutoCompleteData(-1L, "...és még " + more + " további intézmény", moreUrl, "more");
        return moreLabeled;
    }

    @Autowired
    public void setInstitutionNameAutoCompleteService(InstitutionNameAutoCompleteService institutionNameAutoCompleteService) {
        this.institutionNameAutoCompleteService = institutionNameAutoCompleteService;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }
}
