/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2010.02.15.
 * Time: 15:54:21
 */
package hu.kozadat.search.presentation.recommendedsearch;

import hu.kozadat.search.model.IndexerSearchResult;
import hu.kozadat.search.service.simplesearch.SimpleSearch;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.service.recommendedsearch.RecommendedSearchService;
import hu.kozadat.search.service.searchstatistic.SearchStatisticService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.LinkedList;
import java.util.List;

@Controller
public class RecommendedSearchController {

    private static final Logger LOGGER = LoggerFactory.getLogger(RecommendedSearchController.class);

    private RecommendedSearchService recommendedSearchService;
    private SearchStatisticService searchStatisticService;
    private SearchUIConfiguration searchUIConfiguration;

    private SimpleSearch simpleSearch;

    @Autowired
    public void setRecommendedSearchService(RecommendedSearchService recommendedSearchService) {
        this.recommendedSearchService = recommendedSearchService;
    }

    @Autowired
    public void setSearchStatisticService(SearchStatisticService searchStatisticService) {
        this.searchStatisticService = searchStatisticService;
    }

    @Autowired
    public void setSimpleSearch(SimpleSearch simpleSearch) {
        this.simpleSearch = simpleSearch;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }

    @RequestMapping("/ajanlott")
    public ModelAndView showRecommendedSearches(HttpServletRequest httpServletRequest, @RequestParam("szoveg") String text) throws Exception {

        String substitutedString = recommendedSearchService.substitute(text);

        // List<String> prevalentSearches = searchStatisticBean.getPrevalentSearches(maxRecommendations);
        int maxRecommendations = searchUIConfiguration.getRecommendedSearchMaxRecommendation();
        int randomInterval = searchUIConfiguration.getRecommendedSearchRandomInterval();
        List<String> prevalentSearches = searchStatisticService.getRandomPrevalentSearches(maxRecommendations, randomInterval);
        List<String> recommendations = new LinkedList<String>();
        if (text == null || text.equals(substitutedString) || substitutedString == null) {
            recommendations.addAll(prevalentSearches);
        } else {
            if (prevalentSearches.contains(substitutedString)) {
                prevalentSearches.remove(substitutedString);
                recommendations.add(substitutedString);
                recommendations.addAll(prevalentSearches);
            } else {
                HttpSession session = httpServletRequest.getSession();
                IndexerSearchResult indexerSearchResult = simpleSearch.search(substitutedString, "__all__", session.getId());
                int resultSize = indexerSearchResult.getResultSize();
                if (resultSize == 0) {
                    recommendations.addAll(prevalentSearches);
                } else {
                    recommendations.add(substitutedString);
                    if (prevalentSearches.size() > maxRecommendations - 1) {
                        recommendations.addAll(prevalentSearches.subList(0, maxRecommendations - 1));
                    } else {
                        recommendations.addAll(prevalentSearches);
                    }
                }
            }
        }

        ModelAndView modelAndView = new ModelAndView("recommendedSearchPart");

        modelAndView.addObject("szoveg", text);
        modelAndView.addObject("recommendations", recommendations);

        return modelAndView;
    }

}
