package hu.kozadat.search.service;

import hu.kozadat.search.persistence.ApplicationConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;

import javax.sql.DataSource;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.05.23.
 * Time: 14:54
 * To change this template use File | Settings | File Templates.
 */
public class SearchUIConfiguration implements SearchUIConfigurationConstants, InitializingBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(SearchUIConfiguration.class);

    private DataSource dataSource;

    private String configName;
    private String configTableName;
    private String configNameColumnName;
    private String configKeyColumnName;
    private String configValueColumnName;

    private ApplicationConfiguration applicationConfiguration;

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public void setConfigName(String configName) {
        this.configName = configName;
    }

    public void setConfigTableName(String configTableName) {
        this.configTableName = configTableName;
    }

    public void setConfigNameColumnName(String configNameColumnName) {
        this.configNameColumnName = configNameColumnName;
    }

    public void setConfigKeyColumnName(String configKeyColumnName) {
        this.configKeyColumnName = configKeyColumnName;
    }

    public void setConfigValueColumnName(String configValueColumnName) {
        this.configValueColumnName = configValueColumnName;
    }

    public void afterPropertiesSet() throws Exception {
        LOGGER.info("Configuration=" + configName);
        LOGGER.info("Connecting to db config...");
        applicationConfiguration = new ApplicationConfiguration(dataSource, configTableName, configNameColumnName, configKeyColumnName, configValueColumnName);

        LOGGER.info("Creating configuration groups...");
        for (String group : GROUPS) {
            applicationConfiguration.addGroup(group);
        }

        LOGGER.info("Configuration groups created.");

        LOGGER.info("Connection to db config established.");
    }

    public String getApplicationHost() {
        return applicationConfiguration.getStringValue(GROUP_GENERAL, KEY_GENERAL_APPLICATION_HOST);
    }

    public String getApplicationContext() {
        return applicationConfiguration.getStringValue(GROUP_GENERAL, KEY_GENERAL_APPLICATION_CONTEXT);
    }

    public String getJdbcDriverUrl() {
        return applicationConfiguration.getStringValue(GROUP_GENERAL, KEY_GENERAL_JDBC_DRIVER_URL);
    }

    public Boolean getSearchCheckCriticalWords() {
        return applicationConfiguration.getBooleanValue(GROUP_SEARCH, KEY_SEARCH_CHECK_CRITICAL_WORDS);
    }

    public int getSearchTextMinLength() {
        return applicationConfiguration.getIntegerValue(GROUP_SEARCH, KEY_SEARCH_SEARCH_TEXT_MIN_LENGTH);
    }

    public int getSearchTextMaxLength() {
        return applicationConfiguration.getIntegerValue(GROUP_SEARCH, KEY_SEARCH_SEARCH_TEXT_MAX_LENGTH);
    }

    public int getSearchResultPageSize() {
        return applicationConfiguration.getIntegerValue(GROUP_SEARCH, KEY_SEARCH_SEARCH_RESULT_PAGE_SIZE);
    }

    public int getRecommendedSearchMaxRecommendation() {
        return applicationConfiguration.getIntegerValue(GROUP_RECOMMENDED_SEARCH, KEY_RECOMMENDED_SEARCH_MAX_RECOMMENDATION);
    }

    public int getRecommendedSearchRandomInterval() {
        return applicationConfiguration.getIntegerValue(GROUP_RECOMMENDED_SEARCH, KEY_RECOMMENDED_SEARCH_RANDOM_INTERVAL);
    }

    public int getInstitutionSearchResultPageSize() {
        return applicationConfiguration.getIntegerValue(GROUP_INSTITUTION_SEARCH, KEY_INSTITUTION_SEARCH_SEARCH_RESULT_PAGE_SIZE);
    }

    public Map<String, String> getGoogleMapsAPIKeyMap() {
        return applicationConfiguration.getGroupEntries(GROUP_GOOGLE_MAPS_API_KEYS);
    }

    public int getInstitutionSearchAutoCompleteLimit() {
        return applicationConfiguration.getIntegerValue(GROUP_INSTITUTION_SEARCH, KEY_INSTITUTION_SEARCH_AUTOCOMPLETE_LIMIT);
    }

    public int getInteractiveSearchMaxResultCount() {
        return applicationConfiguration.getIntegerValue(GROUP_INTERACTIVE_SEARCH, KEY_INTERACTIVE_SEARCH_MAX_RESULT_COUNT);
    }

    public int getSimpleSearchMaxResultCount() {
        return applicationConfiguration.getIntegerValue(GROUP_SIMPLE_SEARCH, KEY_SIMPLE_SEARCH_MAX_RESULT_COUNT);
    }

    public int getWebServiceSearchMaxResultCount() {
        return applicationConfiguration.getIntegerValue(GROUP_SIMPLE_SEARCH, KEY_SIMPLE_SEARCH_WS_MAX_RESULT_COUNT);
    }

    public String getAttachmentContentRoot() {
        return applicationConfiguration.getStringValue(GROUP_ATTACHMENT_VIEWER, KEY_ATTACHMENT_VIEWER_ATTACHMENT_CONTENT_ROOT_DIR);
    }
}
