/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2010.02.12.
 * Time: 16:06:27
 */
package hu.kozadat.search.service.recommendedsearch;

import hu.kozadat.search.exception.DictionarySearcherException;
import hu.kozadat.search.exception.TermParseException;
import hu.kozadat.search.persistence.recommendedsearch.CriticalWordSearcher;
import hu.kozadat.search.persistence.recommendedsearch.DictionarySearcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service
public class RecommendedSearchService implements InitializingBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(RecommendedSearchService.class);

    private CriticalWordSearcher criticalWordSearcher;
    private DictionarySearcher substituteWordSearcher;

    @Autowired
    @Qualifier("criticalWordSearcher")
    public void setCriticalWordSearcher(CriticalWordSearcher criticalWordSearcher) {
        this.criticalWordSearcher = criticalWordSearcher;
    }

    @Autowired
    @Qualifier("substituteWordSearcher")
    public void setSubstituteWordSearcher(DictionarySearcher substituteWordSearcher) {
        this.substituteWordSearcher = substituteWordSearcher;
    }

    public void afterPropertiesSet() throws Exception {
        try {
            this.criticalWordSearcher.init();
            this.substituteWordSearcher.init();
        } catch (DictionarySearcherException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        }

    }

    public boolean containsCriticalWord(String queryString) throws DictionarySearcherException {
        try {
            return !criticalWordSearcher.search(queryString).isEmpty();
        } catch (TermParseException e) {
            LOGGER.error("containsCriticalWord(): " + e.getMessage() + ", query=" + queryString);
            return true;
        }
    }

    public String substitute(String queryString) throws DictionarySearcherException {
        try {
            Map<String, List<String>> map = substituteWordSearcher.fuzzySearch(queryString);
            String substitutedQueryString = queryString;
            for (String term : map.keySet()) {
                if (map.get(term) != null && !map.get(term).isEmpty()) {
                    substitutedQueryString = queryString.replaceAll(term, map.get(term).get(0));
                }
            }
            return substitutedQueryString;
        } catch (TermParseException e) {
            LOGGER.error("substitute(): " + e.getMessage() + ", query=" + queryString);
            return null;
        }
    }

}
