package hu.kozadat.search.ui;

import hu.kozadat.search.Utils;
import hu.kozadat.search.base.renderer.AbstractResultListItemRenderer;
import hu.kozadat.search.model.ResultItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;

/**
 * User: codex
 */
@Deprecated
public class TemplateBasedRenderer extends AbstractResultListItemRenderer {

    private Logger LOGGER = LoggerFactory.getLogger(TemplateBasedRenderer.class);

    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    public static final String QUALIFIER_SEPARATOR = "-";
    public static final String RENDERER_CONFIG_DEFAULT = "default";

    public static final String SUBS_KEY_INDEX = "idx";
    public static final String SUBS_KEY_SCHEMA_NAME = "schemaName";
    public static final String SUBS_KEY_PROVIDER_NAME = "providerName";
    public static final String SUBS_KEY_PROVIDER_URL = "providerUrl";
    public static final String SUBS_KEY_MODIFICATION_DATE = "modified";
    public static final String SUBS_KEY_URL = "url";
    public static final String SUBS_KEY_DETAIL_URL = "detailUrl";
    public static final String SUBS_KEY_MORE_RECORDS_URL = "moreRecordsUrl";

    public static final String NO_VALUE = "";

    public static final int TEMPLATE_TEXT_SEARCH_NORMAL = 0;
    public static final int TEMPLATE_TEXT_SEARCH_MORERECORDS = 1;

    private Map substitutionMap = new HashMap();
    private RendererConfig rendererConfig;
    private TemplateCache templateCache;
    private int usedTemplate;

    public TemplateBasedRenderer() {
        super();
        setElementConfig("title", "title");
        setElementConfig("keyword", "subject/keyword");
        setElementConfig("creator", "creator");
        setElementConfig("description", "description");
        setElementConfig("date", "date/available");
        setElementConfig("url", "source");
    }

    public void setRendererConfig(RendererConfig rendererConfig) {
        this.rendererConfig = rendererConfig;
    }

    public void setTemplateCache(TemplateCache templateCache) {
        this.templateCache = templateCache;
    }

    public void processLine(String key, String qualifier, String value, boolean containsHighlight) {
        if ("title".equals(key) && (value == null || "".equals(value)))
            return;

        if (qualifier != null && !"".equals(qualifier))
            substitutionMap.put(key.toLowerCase() + QUALIFIER_SEPARATOR + qualifier.toLowerCase(), value);
        else
            substitutionMap.put(key.toLowerCase(), value);
    }

    private String getTemplate(String schema) {
        String res;

        RendererConfigEntry cfg = (RendererConfigEntry) rendererConfig.getRendererMap().get(schema);
        if (cfg == null)
            cfg = (RendererConfigEntry) rendererConfig.getRendererMap().get(RENDERER_CONFIG_DEFAULT);

        switch (usedTemplate) {
            case TEMPLATE_TEXT_SEARCH_MORERECORDS: {
                res = cfg.getMoreRecordsEntryTemplate();
                break;
            }
            case TEMPLATE_TEXT_SEARCH_NORMAL:
            default: {
                res = cfg.getNormalEntryTemplate();
                break;
            }
        }

        return templateCache.getTemplate(res);
    }


    private void addCustomSubstitutions(ResultItem item, int resultIndex) {
        //-- index
        substitutionMap.put(SUBS_KEY_INDEX, String.valueOf(resultIndex));

        //-- url
        String source = getInternalValue("url");
        if (source != null)
            item.setUrl(source);

        // String url = getResponse().encodeURL(getUrlPrefix() + "?id=" + resultIndex + "&dpId=" + item.getDataProviderId());
//        substitutionMap.put( SUBS_KEY_URL, url );
        substitutionMap.put(SUBS_KEY_URL, source);

        //-- schema
        substitutionMap.put(SUBS_KEY_SCHEMA_NAME, item.getMetadataType());

        //-- provider name
        substitutionMap.put(SUBS_KEY_PROVIDER_NAME, item.getDataProvider());

        //-- provider URL
        substitutionMap.put(SUBS_KEY_PROVIDER_URL, "institutioninfo.html?institutionId=" + item.getDataProviderId());

        //-- detail url
        String detailUrl = "showrecord.html?id=" + resultIndex;
        if (usedTemplate == TEMPLATE_TEXT_SEARCH_MORERECORDS)
            detailUrl += "&dp=true&institutionId=" + item.getDataProviderId();
        substitutionMap.put(SUBS_KEY_DETAIL_URL, detailUrl);

        //-- modification date
        if (item.getModificationDate() != null)
            substitutionMap.put(SUBS_KEY_MODIFICATION_DATE, sdf.format(item.getModificationDate()));
        else
            substitutionMap.put(SUBS_KEY_MODIFICATION_DATE, "");

        //-- more records url
        String moreRecordsUrl = "dpresults.html?institutionId=" + item.getDataProviderId();
        substitutionMap.put(SUBS_KEY_MORE_RECORDS_URL, moreRecordsUrl);
    }

    public String getResultHtmlFragment(ResultItem item, int resultIndex) {
        //-- Prepare substition map and do the job
        addCustomSubstitutions(item, resultIndex);

        String res = Utils.replacePlaceHolders(getTemplate(item.getMetadataType()), substitutionMap, NO_VALUE);

        return res;
    }

    public ResultItem getResultItemFor(String resultText, int resultIndex, HttpServletResponse response) {
        usedTemplate = TEMPLATE_TEXT_SEARCH_NORMAL;
        ResultItem res = super.getResultItemFor(resultText, resultIndex, response);
        substitutionMap.clear();

        return res;
    }

    public ResultItem getResultItemFor(String resultText, int resultIndex, HttpServletResponse response, int template) {
        usedTemplate = template;
        ResultItem res = super.getResultItemFor(resultText, resultIndex, response);
        substitutionMap.clear();

        return res;
    }
}
