package hu.kozadat.search.web;

import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.filter.CharacterEncodingFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Collections;
import java.util.Set;
import java.util.TreeSet;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2013.04.16.
 * Time: 16:12
 * To change this template use File | Settings | File Templates.
 */
public class SearchUIEncodingFilter extends CharacterEncodingFilter {

    private static final String REQUEST_PARAMETER_ENCODING = "encoding";

    private String dynamicEncodingAwareUrlPrefixList;
    private Set<String> dynamicEncodingAwareUrlPrefixes = new TreeSet<>();

    public void setDynamicEncodingAwareUrlPrefixList(String dynamicEncodingAwareUrlPrefixList) {
        this.dynamicEncodingAwareUrlPrefixList = dynamicEncodingAwareUrlPrefixList;
        if (this.dynamicEncodingAwareUrlPrefixList != null) {
            dynamicEncodingAwareUrlPrefixes.clear();
            String[] prefixes = dynamicEncodingAwareUrlPrefixList.split(",");
            Collections.addAll(dynamicEncodingAwareUrlPrefixes, prefixes);
        }
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String servletPath = request.getServletPath();

        boolean dynamicEncoding = false;
        for (String prefix : dynamicEncodingAwareUrlPrefixes) {
            if (servletPath.startsWith(prefix)) {
                dynamicEncoding = true;
            }
        }

        if (dynamicEncoding) {
            String encoding = ServletRequestUtils.getStringParameter(request, REQUEST_PARAMETER_ENCODING, null);
            if (encoding == null) {
                encoding = (String) request.getAttribute("encoding");
            }

            if (encoding != null) {
                request.setCharacterEncoding(encoding);
                response.setCharacterEncoding(encoding);
            }

            filterChain.doFilter(request, response);

        } else {
            super.doFilterInternal(request, response, filterChain);
        }

    }

}
