/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2009.12.18.
 * Time: 16:49:12
 */
package hu.kozadat.search.web.servlet;

import hu.kozadat.search.persistence.DataProviderOperations;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.web.map.*;
import hu.scriptum.einfoszab.api.EInfoSzabOperationException;
import hu.scriptum.einfoszab.datamodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;


public class DataProviderMapServlet extends HttpServlet {

    private static final Logger LOGGER = LoggerFactory.getLogger(DataProviderMapServlet.class);

    private static final String GOOGLE_MAPS_CONTEXT = "hu.kozadat.search.web.map";
    private static final String XML_DECLARATION = "com.sun.xml.bind.xmlDeclaration";

    private static final String KML_MIME_TYPE = "application/vnd.google-earth.kml+xml";

    private static final String COORDINATES_BUDAPEST_HUNGARY = "19.040758,47.498406,0";
    private static final String COORDINATES_CENTER_HUNGARY = "19.503479,47.180087";

    private static final String DEFAULT_COORDINATES = COORDINATES_BUDAPEST_HUNGARY;
    private static final String DEFAULT_PLACEMARK_NAME = "K\u00F6zadat program";
    private static final int SETTLEMENT_WITH_DISTINCT_ID = 133;
    private static final String KML_ENCODING = "UTF-8";

    private static final String ANCHOR_MSG_SELECT_SUBREGION = "Kist\u00E9rs\u00E9g kiv\u00E1laszt\u00E1sa";
    private static final String ANCHOR_MSG_SELECT_DATAPROVIDER = "Adatgazda kiv\u00E1laszt\u00E1sa";
    private static final String BALLOONSTYLE_TEXT = "<h3>$[name]</h3>$[description]";
    private static final String EINFOSZAB_BALLOON_STYLE_ID = "einfoszabBalloonStyle";
    private static final String EINFOSZAB_STYLE_URL = "#einfoszabBalloonStyle";
    private static final String URL_MACRO_DATA_PROVIDER_ID = "{dataProviderId}";
    private static final String URL_MACRO_SUBREGION_ID = "{subRegionId}";

    private DataProviderOperations dataProviderOperations;
    private SearchUIConfiguration searchUIConfiguration;

    private String institutionInfoBaseUrl;
    private String subRegionBaseUrl;

    private void handleRequestInternal(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        LOGGER.debug("handleRequestInternal() CALLED");

        try {

            long regionId = ServletRequestUtils.getLongParameter(httpServletRequest, "regionId", -1L);
            Region region = dataProviderOperations.getRegion(regionId);
            boolean regionDisplayMode = region != null;

            long countyId = ServletRequestUtils.getLongParameter(httpServletRequest, "countyId", -1L);
            County county = dataProviderOperations.getCounty(countyId);
            boolean countyDisplayMode = county != null;

            long subRegionId = ServletRequestUtils.getLongParameter(httpServletRequest, "subRegionId", -1L);
            SubRegion subRegion = dataProviderOperations.getSubRegion(subRegionId);

            boolean subRegionDisplayMode = subRegion != null;

            String layout = ServletRequestUtils.getStringParameter(httpServletRequest, "layout", subRegionDisplayMode ? "subRegion" : "dataProvider");
            MapLayout mapLayout = MapLayout.getLayout(layout);

            LOGGER.debug("creating kml response...");

            DataProvider dataProvider = null;
            long dataProviderId = ServletRequestUtils.getLongParameter(httpServletRequest, "dataProviderId", -1L);
            if (dataProviderId != -1) {
                dataProvider = dataProviderOperations.getDataProvider(dataProviderId);
            }

            Object response;

            if ("encodingtest".equals(layout)) {
                LOGGER.debug("mode: TEST");
                response = createEncodingTestKml();
            } else if (regionDisplayMode) {
                LOGGER.debug("mode: REGION");
                if (MapLayout.Region.equals(mapLayout)) {
                    response = createRegionKml(region);
                } else {
                    response = createDefaultKml();
                }
            } else if (countyDisplayMode) {
                LOGGER.debug("mode: COUNTY");
                if (MapLayout.County.equals(mapLayout)) {
                    response = createCountyKml(county);
                } else {
                    response = createDefaultKml();
                }
            } else if (subRegionDisplayMode && dataProvider == null) {
                LOGGER.debug("mode: SUBREGION");
                if (MapLayout.SubRegion.equals(mapLayout)) {
                    response = createSubRegionKml(subRegion);
                } else {
                    response = createDefaultKml();
                }
            } else if (subRegionDisplayMode && dataProvider != null) {
                LOGGER.debug("mode: SUBREGION with DataProvider");
                if (MapLayout.SubRegion.equals(mapLayout)) {
                    response = createSubRegionKml(subRegion);
                } else if (MapLayout.County.equals(mapLayout)) {
                    response = createCountyKml(dataProvider, subRegion.getCounty());
                } else if (MapLayout.Region.equals(mapLayout)) {
                    response = createRegionKml(dataProvider, subRegion.getCounty().getRegion());
                } else {
                    response = createDefaultKml();
                }
            } else if (dataProvider != null) {
                LOGGER.debug("mode: DATAPROVIDER");
                Address address = dataProviderOperations.getAddress(dataProvider.getId());
                LOGGER.debug("address=" + address);

                if (MapLayout.District.equals(mapLayout)) {
                    response = createDistrictKml(address);
                } else if (MapLayout.Settlement.equals(mapLayout)) {
                    response = createSettlementKml(address);
                } else if (MapLayout.SubRegion.equals(mapLayout)) {
                    response = createSubRegionKml(address.getZipCode().getSettlement().getSubRegion());
                } else if (MapLayout.County.equals(mapLayout)) {
                    response = createCountyKml(dataProvider, address.getZipCode().getSettlement().getSubRegion().getCounty());
                } else if (MapLayout.Region.equals(mapLayout)) {
                    response = createRegionKml(dataProvider, address.getZipCode().getSettlement().getSubRegion().getCounty().getRegion());
                } else {
                    response = createDataProviderKml(dataProvider, address);
                }
            } else {
                response = createDefaultKml();
            }


            JAXBContext jaxbContext = JAXBContext.newInstance(GOOGLE_MAPS_CONTEXT);
            Marshaller marshaller = jaxbContext.createMarshaller();
            marshaller.setProperty(XML_DECLARATION, Boolean.TRUE);
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);


            httpServletResponse.setContentType(KML_MIME_TYPE);
            httpServletResponse.setCharacterEncoding(KML_ENCODING);
            httpServletResponse.setStatus(HttpServletResponse.SC_OK);
            PrintWriter printWriter = httpServletResponse.getWriter();

            marshaller.marshal(response, printWriter);
            LOGGER.debug("kml response created and sent");
        } catch (JAXBException e) {
            LOGGER.debug(e.getMessage(), e);
            throw new ServletException("Google KML creation failed: " + e.getMessage(), e);
        } catch (EInfoSzabOperationException e) {
            LOGGER.debug(e.getMessage(), e);
            throw new ServletException("Google KML creation failed: " + e.getMessage(), e);
        }

        LOGGER.debug("handleRequestInternal() FINISHED");
    }

    private CountyKml createCountyKml(DataProvider dataProvider, County county) {
        long start = System.currentTimeMillis();
        LOGGER.debug("creating county KML...");

        CountyKml countyKml = new CountyKml();

        if (county != null) {
            long countyId = county.getId();
            List<SubRegion> subRegions = dataProviderOperations.getSubRegionsInCounty(countyId);
            Map<Long, Long> map = dataProviderOperations.getDataProviderCountInSubRegionsByCounty(countyId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (SubRegion subRegion : subRegions) {
                Point point = new Point();

                point.setCoordinates(new StringBuilder().append(subRegion.getLongitude()).append(",").append(subRegion.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(subRegion.getName() + " kist\u00E9rs\u00E9g");

                String url = getSelectDataProviderOrSubRegionUrl(dataProvider, subRegion);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_SUBREGION).append("</a>");
                description.append("<p><br/>").append("Adatgazd\u00E1k sz\u00E1ma a kist\u00E9rs\u00E9gben: ").append(map.containsKey(subRegion.getId()) ? map.get(subRegion.getId()) : 0).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            countyKml.setDocument(document);
        }

        LOGGER.debug("county KML created in " + (System.currentTimeMillis() - start) + " ms");

        return countyKml;
    }

    private CountyKml createCountyKml(County county) {
        long start = System.currentTimeMillis();
        LOGGER.debug("creating county KML...");

        CountyKml countyKml = new CountyKml();

        if (county != null) {
            long countyId = county.getId();
            List<SubRegion> subRegions = dataProviderOperations.getSubRegionsInCounty(countyId);
            Map<Long, Long> map = dataProviderOperations.getDataProviderCountInSubRegionsByCounty(countyId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (SubRegion subRegion : subRegions) {
                Point point = new Point();

                point.setCoordinates(new StringBuilder().append(subRegion.getLongitude()).append(",").append(subRegion.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(subRegion.getName() + " kist\u00E9rs\u00E9g");

                String url = getSelectSubRegionUrl(subRegion);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_SUBREGION).append("</a>");
                description.append("<p><br/>").append("Adatgazd\u00E1k sz\u00E1ma a kist\u00E9rs\u00E9gben: ").append(map.containsKey(subRegion.getId()) ? map.get(subRegion.getId()) : 0).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            countyKml.setDocument(document);
        }

        LOGGER.debug("county KML created in " + (System.currentTimeMillis() - start) + " ms");

        return countyKml;
    }

    private DataProviderKml createDataProviderKml(DataProvider dataProvider, Address address) throws EInfoSzabOperationException {
        LOGGER.debug("creating dataprovider KML...");

        Document document = new Document();

        List<Placemark> placemarks = new LinkedList<Placemark>();
        document.setPlacemarks(placemarks);

        Style style = new Style();
        style.setId(EINFOSZAB_BALLOON_STYLE_ID);
        BalloonStyle balloonStyle = new BalloonStyle();
        balloonStyle.setText(BALLOONSTYLE_TEXT);
        style.setBalloonStyle(balloonStyle);

        document.setStyle(style);

        Point point = new Point();
        if (address != null) {
            point.setCoordinates(new StringBuilder().append(address.getLongitude()).append(",").append(address.getLatitude()).append(",").append("0").toString());
        } else {
            point.setCoordinates(COORDINATES_BUDAPEST_HUNGARY);
        }

        Placemark placemark = new Placemark();
        placemark.setName(dataProvider.getInstitution().getName());

        StringBuilder description = new StringBuilder();
        if (address == null) {
            description.append("Az adatgazda c\u00EDme nincs megfelel\u0151en megadva");
        } else {
            description.append(address.getZipCode().getZipCode()).append(" ");
            description.append(address.getZipCode().getSettlement().getName()).append(" <br />");
            description.append(address.getAddress());
        }

        placemark.setDescription(description.toString());
        placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
        placemark.setPoint(point);

        placemarks.add(placemark);

        DataProviderKml dataProviderKml = new DataProviderKml();
        dataProviderKml.setDocument(document);

        LOGGER.debug("dataprovider KML created");

        return dataProviderKml;
    }

    private Kml createDefaultKml() {
        LOGGER.debug("creating default KML...");

        Point point = new Point();
        point.setCoordinates(DEFAULT_COORDINATES);

        Placemark placemark = new Placemark();
        placemark.setName(DEFAULT_PLACEMARK_NAME);
        placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
        placemark.setPoint(point);

        Kml kml = new Kml();
        kml.setPlacemark(placemark);

        LOGGER.debug("default KML created");

        return kml;
    }

    private DistrictKml createDistrictKml(Address address) {
        LOGGER.debug("creating district KML...");

        DistrictKml districtKml = new DistrictKml();
        if (address != null && address.getZipCode().getSettlement().getSubRegion().getId() == SETTLEMENT_WITH_DISTINCT_ID) {
            long districtId = address.getZipCode().getSettlement().getId();
            List<DataProvider> dataProviders = dataProviderOperations.getDataProvidersInSettlement(districtId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (DataProvider dataProvider : dataProviders) {
                Point point = new Point();

                Address dataProviderAddress = dataProvider.getInstitution().getNewAddress();
                point.setCoordinates(new StringBuilder().append(dataProviderAddress.getLongitude()).append(",").append(dataProviderAddress.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(dataProvider.getInstitution().getName());

                String url = getSelectDataProviderOrSubRegionUrl(dataProvider, null);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_DATAPROVIDER).append("</a>");
                description.append("<p><br/>");
                description.append(dataProviderAddress.getZipCode().getZipCode()).append(" ");
                description.append(dataProviderAddress.getZipCode().getSettlement().getName()).append(" <br />");
                description.append(dataProviderAddress.getAddress()).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            districtKml.setDocument(document);
        }

        LOGGER.debug("district KML created");

        return districtKml;
    }

    private Kml createEncodingTestKml() {
        LOGGER.debug("creating encoding test KML...");

        Point point = new Point();
        point.setCoordinates(DEFAULT_COORDINATES);

        Placemark placemark = new Placemark();
        placemark.setName("\u00E1rv\u00EDzt\u0171r\u0151 t\u00FCk\u00F6rf\u00FAr\u00F3g\u00E9p");
        placemark.setPoint(point);

        Kml kml = new Kml();
        kml.setPlacemark(placemark);

        LOGGER.debug("encoding test KML created");

        return kml;
    }

    private RegionKml createRegionKml(DataProvider dataProvider, Region region) {
        long start = System.currentTimeMillis();
        LOGGER.debug("creating region KML...");

        RegionKml regionKml = new RegionKml();

        if (region != null) {
            long regionId = region.getId();
            List<SubRegion> subRegions = dataProviderOperations.getSubRegionsInRegion(regionId);
            Map<Long, Long> map = dataProviderOperations.getDataProviderCountInSubRegionsByRegion(regionId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (SubRegion subRegion : subRegions) {
                Point point = new Point();

                point.setCoordinates(new StringBuilder().append(subRegion.getLongitude()).append(",").append(subRegion.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(subRegion.getName() + " kist\u00E9rs\u00E9g");

                String url = getSelectDataProviderOrSubRegionUrl(dataProvider, subRegion);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_SUBREGION).append("</a>");
                description.append("<p><br/>").append("Adatgazd\u00E1k sz\u00E1ma a kist\u00E9rs\u00E9gben: ").append(map.containsKey(subRegion.getId()) ? map.get(subRegion.getId()) : 0).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            regionKml.setDocument(document);
        }

        LOGGER.debug("region KML created in " + (System.currentTimeMillis() - start) + " ms");

        return regionKml;
    }

    private RegionKml createRegionKml(Region region) {
        long start = System.currentTimeMillis();
        LOGGER.debug("creating region KML...");

        RegionKml regionKml = new RegionKml();

        if (region != null) {
            long regionId = region.getId();
            List<SubRegion> subRegions = dataProviderOperations.getSubRegionsInRegion(regionId);
            Map<Long, Long> map = dataProviderOperations.getDataProviderCountInSubRegionsByRegion(regionId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (SubRegion subRegion : subRegions) {
                Point point = new Point();

                point.setCoordinates(new StringBuilder().append(subRegion.getLongitude()).append(",").append(subRegion.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(subRegion.getName() + " kist\u00E9rs\u00E9g");

                String url = getSelectSubRegionUrl(subRegion);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_SUBREGION).append("</a>");
                description.append("<p><br/>").append("Adatgazd\u00E1k sz\u00E1ma a kist\u00E9rs\u00E9gben: ").append(map.containsKey(subRegion.getId()) ? map.get(subRegion.getId()) : 0).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            regionKml.setDocument(document);
        }

        LOGGER.debug("region KML created in " + (System.currentTimeMillis() - start) + " ms");

        return regionKml;
    }

    private SettlementKml createSettlementKml(Address address) {
        LOGGER.debug("creating settlement KML...");

        SettlementKml settlementKml = new SettlementKml();
        if (address != null) {
            ZipCode zipCode = address.getZipCode();
            Settlement settlement = zipCode.getSettlement();
            Long settlementId = settlement.getId();
            List<DataProvider> dataProviders = dataProviderOperations.getDataProvidersInSettlement(settlementId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (DataProvider dataProvider : dataProviders) {
                Point point = new Point();

                Address dataProviderAddress = dataProvider.getInstitution().getNewAddress();
                point.setCoordinates(new StringBuilder().append(dataProviderAddress.getLongitude()).append(",").append(dataProviderAddress.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(dataProvider.getInstitution().getName());

                String url = getSelectDataProviderOrSubRegionUrl(dataProvider, null);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_DATAPROVIDER).append("</a>");
                description.append("<p><br/>");
                description.append(dataProviderAddress.getZipCode().getZipCode()).append(" ");
                description.append(dataProviderAddress.getZipCode().getSettlement().getName()).append("<br />");
                description.append(dataProviderAddress.getAddress()).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            settlementKml.setDocument(document);
        }

        LOGGER.debug("settlement KML created");

        return settlementKml;
    }

    private SubRegionKml createSubRegionKml(SubRegion subRegion) {
        LOGGER.debug("creating subregion KML...");

        SubRegionKml subRegionKml = new SubRegionKml();
        if (subRegion != null) {
            long subRegionId = subRegion.getId();
            List<DataProvider> dataProviders = dataProviderOperations.getDataProvidersInSubRegion(subRegionId);
            List<Placemark> placemarks = new LinkedList<Placemark>();
            Document document = new Document();

            BalloonStyle balloonStyle = new BalloonStyle();
            balloonStyle.setText(BALLOONSTYLE_TEXT);

            Style style = new Style();
            style.setId(EINFOSZAB_BALLOON_STYLE_ID);
            style.setBalloonStyle(balloonStyle);
            document.setStyle(style);

            document.setPlacemarks(placemarks);
            for (DataProvider dataProvider : dataProviders) {
                Point point = new Point();

                Address dataProviderAddress = dataProvider.getInstitution().getNewAddress();
                point.setCoordinates(new StringBuilder().append(dataProviderAddress.getLongitude()).append(",").append(dataProviderAddress.getLatitude()).append(",").append("0").toString());

                Placemark placemark = new Placemark();
                placemark.setName(dataProvider.getInstitution().getName());

                String url = getSelectDataProviderOrSubRegionUrl(dataProvider, null);

                StringBuilder description = new StringBuilder();
                description.append("<a href=\"").append(url).append("\">").append(ANCHOR_MSG_SELECT_DATAPROVIDER).append("</a>");
                description.append("<p><br/>");
                description.append(dataProviderAddress.getZipCode().getZipCode()).append(" ");
                description.append(dataProviderAddress.getZipCode().getSettlement().getName()).append("<br />");
                description.append(dataProviderAddress.getAddress()).append("</p>");

                placemark.setDescription(description.toString());
                placemark.setStyleUrl(EINFOSZAB_STYLE_URL);
                placemark.setPoint(point);

                placemarks.add(placemark);
            }
            subRegionKml.setDocument(document);
        }

        LOGGER.debug("subregion KML created");

        return subRegionKml;
    }

    @Override
    public void destroy() {
        super.destroy();
        LOGGER.debug("destroy() CALLED");

        dataProviderOperations = null;

        LOGGER.debug("destroy() FINISHED");
    }

    @Override
    protected void doGet(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        handleRequestInternal(httpServletRequest, httpServletResponse);
    }

    @Override
    protected void doPost(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
        handleRequestInternal(httpServletRequest, httpServletResponse);
    }

    private String getSelectDataProviderOrSubRegionUrl(DataProvider dataProvider, SubRegion subRegion) {
        StringBuilder selectDataProviderOrSubRegionUrl = new StringBuilder(institutionInfoBaseUrl);

        int startIndex = selectDataProviderOrSubRegionUrl.indexOf(URL_MACRO_DATA_PROVIDER_ID);
        selectDataProviderOrSubRegionUrl.replace(startIndex, startIndex + URL_MACRO_DATA_PROVIDER_ID.length(), Long.toString(dataProvider.getId()));

        if (subRegion != null) {
            selectDataProviderOrSubRegionUrl.append("?").append("kisterseg").append("=").append(subRegion.getId());
        }

        return selectDataProviderOrSubRegionUrl.toString();
    }

    private String getSelectSubRegionUrl(SubRegion subRegion) {
        StringBuilder selectSubRegionUrl = new StringBuilder(subRegionBaseUrl);

        int startIndex = selectSubRegionUrl.indexOf(URL_MACRO_SUBREGION_ID);
        selectSubRegionUrl.replace(startIndex, startIndex + URL_MACRO_SUBREGION_ID.length(), Long.toString(subRegion.getId()));

        return selectSubRegionUrl.toString();
    }

    @Override
    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);
        LOGGER.debug("init() CALLED");

        WebApplicationContext context = WebApplicationContextUtils.getWebApplicationContext(getServletContext());

        dataProviderOperations = context.getBean("dataProviderOperations", DataProviderOperations.class);
        searchUIConfiguration = context.getBean("searchuiConfiguration", SearchUIConfiguration.class);

        String host = searchUIConfiguration.getApplicationHost();
        String contextPath = searchUIConfiguration.getApplicationContext();

        institutionInfoBaseUrl = host + contextPath + "/intezmeny/adatlap/{dataProviderId}";
        subRegionBaseUrl = host + contextPath + "/kisterseg/{subRegionId}";

        LOGGER.debug("init() FINISHED");
    }
}
