package hu.kozadat.search.ws;

import hu.kozadat.search.base.preprocessor.QueryPreprocessorManager;
import hu.kozadat.search.base.preprocessor.QueryPreprocessorResult;
import hu.kozadat.search.model.IndexerSearchResultItem;
import hu.kozadat.search.service.IndexerSearchResultParser;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.scriptum.nda.api.MDcSearch;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: codex
 * Date: May 6, 2008
 * Time: 1:30:39 AM
 * To change this template use File | Settings | File Templates.
 */
@Service("wsSimpleSearch")
public class WSSimpleSearch implements InitializingBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(WSSimpleSearch.class);

    private static final String WS_SIMPLE_SEARCH_SESSION_ID = "WSSession";

    private SearchUIConfiguration searchUIConfiguration;
    private MDcSearch searchService;
    private int maxResultCount = 200;
    private QueryPreprocessorManager queryPreprocessor;


    public Collection<SearchResultItem> search(String query, Integer schemaId) {

        QueryPreprocessorResult queryPreprocessorResult = queryPreprocessor.preprocess(query != null ? query.toLowerCase() : query, true);
        String searchExpression = queryPreprocessorResult.getFilteredQuery();

        if (schemaId != null) {
            StringBuilder searchExpressionBuilder = new StringBuilder();

            searchExpressionBuilder.append("(");
            searchExpressionBuilder.append(searchExpression);
            searchExpressionBuilder.append(")");
            searchExpressionBuilder.append(" AND ");
            searchExpressionBuilder.append(MDcSearch.FIELD_SCHEMA_ID);
            searchExpressionBuilder.append(":");
            searchExpressionBuilder.append(MDcSearch.ID_PREFIX);
            searchExpressionBuilder.append(String.valueOf(schemaId));
            searchExpressionBuilder.append(MDcSearch.ID_POSTFIX);

            searchExpression =  searchExpressionBuilder.toString();
        }

        List<SearchResultItem> result = new LinkedList<SearchResultItem>();

        int resultSize = searchService.executeQuery(WS_SIMPLE_SEARCH_SESSION_ID, searchExpression);

        IndexerSearchResultParser indexerSearchResultParser = new IndexerSearchResultParser();

        if (resultSize > 0) {
            String[] hits = searchService.getHits(WS_SIMPLE_SEARCH_SESSION_ID, 0, Math.max(resultSize, maxResultCount));

            for (String hit : hits) {
                IndexerSearchResultItem indexerSearchResultItem = indexerSearchResultParser.parse(hit);

                List<ElementItem> elementItems = new LinkedList<ElementItem>();
                SearchResultItem item = new SearchResultItem(indexerSearchResultItem.getDataProviderId(), elementItems);

                for (Map.Entry<String, Map<String, List<String>>> schemaElement : indexerSearchResultItem.getSchemaElements().entrySet()) {
                    String name = schemaElement.getKey();
                    for (Map.Entry<String, List<String>> schemaQualifier : schemaElement.getValue().entrySet()) {
                        String qualifier = schemaQualifier.getKey();
                        for (String value : schemaQualifier.getValue()) {
                            ElementItem elementItem = new ElementItem(name, qualifier, value);
                            elementItems.add(elementItem);
                        }
                    }
                }

                result.add(item);

            }
        }

        searchService.closeSession(WS_SIMPLE_SEARCH_SESSION_ID);

        return result;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }

    @Autowired
    public void setQueryPreprocessor(QueryPreprocessorManager queryPreprocessor) {
        this.queryPreprocessor = queryPreprocessor;
    }

    @Autowired
    public void setSearchService(MDcSearch searchService) {
        this.searchService = searchService;
    }

    public void afterPropertiesSet() throws Exception {
        this.maxResultCount = searchUIConfiguration.getWebServiceSearchMaxResultCount();
    }
}



